<?php

/**
 * @file
 * Provide a custom views sort data that isn't tied to any other module.
 */

use Drupal\field\Entity\FieldConfig;
use Drupal\Core\Database\Database;

/**
 * Implements hook_views_data().
 */
function views_order_by_delta_views_data() {
  $data = [];
  $entity_type_manager = \Drupal::entityTypeManager();
  $entity_field_manager = \Drupal::service('entity_field.manager');

  // Iterate over all entity types but consider only the ones
  // that are used by views.
  foreach ($entity_type_manager->getDefinitions() as $entity_type_id => $entity_type) {
    if (!$entity_type->hasHandlerClass('views_data')) {
      continue;
    }
    foreach ($entity_field_manager->getFieldStorageDefinitions($entity_type_id) as $field) {
      if ($field->getType() != 'entity_reference') {
        continue;
      }

      // Get the table name and verify if this table has a delta column.
      $storage = $entity_type_manager->getStorage($entity_type_id);
      $table_name = $storage->getTableMapping()->getDedicatedDataTableName($field);
      $schema = Database::getConnection()->schema();
      // This if is probably not needed since all references should have
      // a delta column.
      if ($schema->fieldExists($table_name, 'delta')) {
        $data[$entity_type->getBaseTable()][$table_name . '__views_order_by_delta'] = [
          'title' => t('Order by delta (using @field)', [
            '@field' => $field->getName(),
          ]),
          'help' => t('Custom sort.'),
          'sort' => [
            'id' => 'order_by_delta',
          ],
          'real field' => 'delta',
        ];
       }
     }
   }

  return $data;
}
